﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using BayesNetwork.Classes;
using Environment.Classes;
using Multimap;
using Extensions;

namespace Environment
{
    public class timmer
    {
        BayesNet bnet;
        int sizeX;
        int sizeY;
        int day = 0;
        Dictionary<string, int> counter = new Dictionary<string, int>();
        double chanceForWater = 0.05;
        // fish can only spawn in water
        double chanceForFish = 0.7;
        double chanceForMountain = 0.1;
        double chanceForTree = 0.3;
        double chanceForRabbit = 0.8;
        double chanceForJackal = 0.2;
        // wolf can only spawn on mountains
        double chanceForWolf = 0.9;

        public timmer()
        {
            setUpBnet();

            // seting up the environment with standard objects
            sizeX = EnvironmentMap.sizeX;
            sizeY = EnvironmentMap.sizeY;
            int X = 0;
            int Y = 0;
            Random rand = new Random(System.DateTime.Now.Millisecond);
            double water;
            double mountain;
            double plant;
            double animal;
            counter.Add("water", 0);
            counter.Add("fish", 0);
            counter.Add("ground", 0);
            counter.Add("mountain", 0);
            counter.Add("tree", 0);
            counter.Add("rabbit", 0);
            counter.Add("jackal", 0);
            counter.Add("wolf", 0);

            while (X < sizeX)
            {
                while (Y < sizeY)
                {
                    water = rand.NextDouble();
                    if (water < chanceForWater)
                    {
                        EnvironmentMap.add(new water(bnet), X, Y);
                        animal = rand.NextDouble();
                        if (animal < chanceForFish)
                            EnvironmentMap.add(new fish(bnet), X, Y);
                    }
                    else
                    {
                        mountain = rand.NextDouble();
                        if (mountain < chanceForMountain)
                        {
                            EnvironmentMap.add(new mountain(bnet), X, Y);
                            animal = rand.NextDouble();
                            if (animal < chanceForWolf)
                                EnvironmentMap.add(new wolf(bnet), X, Y);
                        }
                        else
                        {
                            EnvironmentMap.add(new ground(bnet), X, Y);
                            plant = rand.NextDouble();
                            if (plant < chanceForTree)
                            {
                                animal = rand.NextDouble();
                                if (animal < chanceForRabbit)
                                    EnvironmentMap.add(new rabbit(bnet), X, Y);
                                EnvironmentMap.add(new tree(bnet), X, Y);
                            }
                            else
                            {
                                animal = rand.NextDouble();
                                if (animal < chanceForJackal)
                                    EnvironmentMap.add(new jackal(bnet), X, Y);
                            }
                        }
                    }
                    ++Y;
                }
                Y = 0;
                ++X;
            }

            //starting infinite loop
            while (true)
            {
                ++day;
                bnet.updateFull();
                clearCounter(counter);

                EnvironmentMap.update(counter);
                Console.WriteLine("Counter :");
                foreach (KeyValuePair<string, int> C in counter)
                {
                    Console.WriteLine(C.Key + " = " + C.Value);
                }
                Console.WriteLine("Day " + day);
                Console.WriteLine("BayesNet :\n");
                Console.WriteLine("Nodes:\n" + bnet.Nodes.toString());
                EnvironmentMap.toString();
                Thread.Sleep(2000);
                Console.WriteLine("\n\n\n\n\n\n\n\n\n\n\n\n\n\n");

            }

        }

        public void clearCounter(Dictionary<string, int> counter)
        {
            counter["water"] = 0;
            counter["fish"] = 0;
            counter["ground"] = 0;
            counter["mountain"] = 0;
            counter["tree"] = 0;
            counter["rabbit"] = 0;
            counter["jackal"] = 0;
            counter["wolf"] = 0;
        }

        public void setUpBnet()
        {
            bnet = new BayesNet();
            EnvironmentMap.setup();

            // set up difrent tempatures
            bnet.addNode("cold", 0.1);
            bnet.addNode("tempermant", 0.4);
            bnet.addNode("warm", 0.3);
            bnet.addNode("hot", 0.1);

            // We now set up wind conditions
            bnet.addNode("gale", 0.1);
            bnet.addNode("windy", 0.2);
            bnet.addNode("breeze", 0.3);

            // Now we set up the world with difrent types of rain, storms, etc

            bnet.addNode("cloudy", 0.6);
            bnet.addNode("raining", 0.2);
            bnet.addNode("snowing", 0.1);
            bnet.addNode("storming", 0.05);
            bnet.addNode("hurrican", 0.05);

            // Now we set up there infcluences

            bnet.addEdge("hot", "cloudy", 0.2, 1);
            bnet.addEdge("cold", "cloudy", 1.7, 1);
            bnet.addEdge("cloudy", "raining", 1.6, 0.1);
            bnet.addEdge("cloudy", "snowing", 1.2, 0.1);
            bnet.addEdge("cloudy", "storming", 1.4, 0.1);
            bnet.addEdge("cloudy", "hurrican", 1.2, 0);
            bnet.addEdge("cold", "snowing", 1.6, 0.2);
            bnet.addEdge("raining", "snowing", 0.4, 1.2);
            bnet.addEdge("raining", "storming", 1, 0);
            bnet.addEdge("raining", "hurrican", 1, 0);
            bnet.addEdge("gale", "hurrican", 1.9, 0.4);
            bnet.addEdge("windy", "storming", 1.6);

        }
    }
}
