﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MicrosoftResearch.Infer.Models;
using MicrosoftResearch.Infer;
//using MicrosoftResearch.Infer.Collections;
//using MicrosoftResearch.Infer.Factors;
//using MicrosoftResearch.Infer.Graphs;
//using MicrosoftResearch.Infer.Transforms;
//using MicrosoftResearch.Infer.Utils;
//using MicrosoftResearch.Infer.Views;
using MicrosoftResearch.Infer.Distributions;
using MicrosoftResearch.Infer.Maths;

namespace testingInfer
{
    class Program
    {
        static void Main(string[] args)
        {
            testBayesNetForward();
        }

        static void coinForwardInference()
        {
            //some simple coin tosses.
            //This tutorial was very straight forward. 
            //Infer probabilitys from other probabilities.
            Variable<bool> firstCoin = Variable.Bernoulli(0.5).Named("Coin1");
            Variable<bool> secondCoin = Variable.Bernoulli(0.5).Named("Coin2");

            Variable<bool> bothHeads = firstCoin & secondCoin;

            InferenceEngine ie = new InferenceEngine();
            //ie.ShowFactorGraph = true;
            Console.WriteLine("Probability both are heads: " + ie.Infer(bothHeads));
            Console.ReadKey();
        }

        static void coinBackwardInference()
        {
            //some simple coin tosses.
            //This tutorial was very straight forward. 
            //Infer probabilitys from other probabilities.
            Variable<bool> firstCoin = Variable.Bernoulli(0.5).Named("Coin1");
            Variable<bool> secondCoin = Variable.Bernoulli(0.5).Named("Coin2");

            Variable<bool> bothHeads = firstCoin & secondCoin;

            InferenceEngine ie = new InferenceEngine();
            bothHeads.ObservedValue = false;
            Console.WriteLine("Chance first coin was heads if both weren't heads: "
                    + ie.Infer(firstCoin));
            Console.ReadKey();
        }

        static void slowMeanThingy()
        {
            //This one is a bit strange.
            //Get a mean from random values, and the precision of the mean I suppose.
            //The second one compiles very fast thou...
            double[] data = new double[100];
            for (int i = 0; i < data.Length; i++) data[i] = Rand.Normal(0, 1);
            Variable<double> mean = Variable.GaussianFromMeanAndVariance(0, 100);
            Variable<double> precision = Variable.GammaFromShapeAndScale(1, 1);
            for (int i = 0; i < data.Length; i++)
            {
                Variable<double> x = Variable.GaussianFromMeanAndPrecision(mean, precision);
                x.ObservedValue = data[i];
            }
            InferenceEngine engine = new InferenceEngine(new VariationalMessagePassing());
            Console.WriteLine("mean=" + engine.Infer(mean));
            Console.WriteLine("prec=" + engine.Infer(precision));
            Console.ReadKey();
        }

        static void fastMeanThingy()
        {
            double[] data = new double[100];
            for (int i = 0; i < data.Length; i++) data[i] = Rand.Normal(0, 1);
            Range dataRange = new Range(data.Length);
            VariableArray<double> x2 = Variable.Array<double>(dataRange);
            Variable<double> mean2 = Variable.GaussianFromMeanAndVariance(0, 100);
            Variable<double> precision2 = Variable.GammaFromShapeAndScale(1, 1);
            x2[dataRange] = Variable.GaussianFromMeanAndPrecision(mean2, precision2).ForEach(dataRange);
            x2.ObservedValue = data;
            InferenceEngine engine2 = new InferenceEngine(new VariationalMessagePassing());
            Console.WriteLine("2, mean=" + engine2.Infer(mean2));
            Console.WriteLine("2, prec=" + engine2.Infer(precision2));
            Console.ReadKey();
        }

        //This would be useful for the agents making predictions on our environment.
        static void testBayesNet()
        {
            //making a bayes network... trying to have rich and poor buy and old people.
            double[] incomes =  { 99, 90, 70, 60, 40, 30 };
            double[] ages = { 50, 18, 60, 30, 40, 25 };
            bool[] willBuy = { true, false, true, false, true, true };

            //create an x vector augmented by 1.
            Vector[] xdata = new Vector[incomes.Length];
            for (int i = 0; i < xdata.Length; i++) xdata[i] 
                = new Vector(incomes[i], ages[i], 1);
            VariableArray<Vector> x = Variable.Observed(xdata).Named("x");

            //create target y
            VariableArray<bool> y = Variable.Observed(willBuy).Named("y");

            Variable<Vector> w = Variable.Random(new VectorGaussian(new Vector(3),
                           PositiveDefiniteMatrix.Identity(3))).Named("w");
            //Range j = y.Range;
            //y[j] = Variable.IsPositive(Variable.InnerProduct(w, x[j]));

            InferenceEngine engine = new InferenceEngine(new ExpectationPropagation());
            VectorGaussian wPosterior = engine.Infer<VectorGaussian>(w);
            Console.WriteLine("Dist over w=\n" + wPosterior);

            VariableArray<bool> ytest = Variable.Array<bool>(new Range(willBuy.Length));
            BayesPointMachine(incomes, ages, Variable.Random(wPosterior), ytest);
            Console.WriteLine("output=\n" + engine.Infer(ytest));
            Console.ReadKey();
        }

        static void BayesPointMachine(
            double[] incomes,
            double[] ages,
            Variable<Vector> w,
            VariableArray<bool> y)
        {
            // Create x vector, augmented by 1
            Range j = y.Range;
            Vector[] xdata = new Vector[incomes.Length];
            for (int i = 0; i < xdata.Length; i++) xdata[i] = new Vector(incomes[i], ages[i], 1);
            VariableArray<Vector> x = Variable.Observed(xdata, j).Named("x");

            // Bayes Point Machine
            y[j] = Variable.IsPositive(Variable.InnerProduct(w, x[j]));
        }

        static void testBayesNetForward()
        {
            Variable<int> temperature = Variable.New<int>().Named("temperature");
            Variable.ConstrainTrue(temperature < 40 & temperature > -20);
            Variable<int> humidity = Variable.New<int>().Named("humidity");
            Variable<bool> cloudy = Variable.Bernoulli(.5).Named("isCloudy");
            Variable<bool> snowing = cloudy & Variable.Bernoulli(.5).Named("isSnowing");
            Variable<bool> raining = cloudy & Variable.Bernoulli(.5).Named("isRaining");

            InferenceEngine ie = new InferenceEngine();

            Console.WriteLine("Chance it is raining(cloudy unknown): " + ie.Infer(raining));
            cloudy.ObservedValue = true;
            Console.WriteLine("Chance it is raining(cloudy): " + ie.Infer(raining));
            Console.WriteLine("Chance it is snowing(cloudy): " + ie.Infer(snowing));
            //temperature.ObservedValue = -2;
            //Console.WriteLine("Chance it is snowing(cloudy): " + ie.Infer(snowing));
            Console.ReadKey();
        }
    }
}
